package com.anagog.anagog_flutter.jedai_sdk

import android.content.Context
import android.content.Intent
import android.util.Log
import com.anagog.jedai.common.poi.config.PoiAlgorithmType1Config
import com.anagog.jedai.core.api.JedAI
import com.anagog.jedai.debugging.JedAIDebugging
import com.anagog.jedai.jema.JedAIJema
import com.anagog.jedai.ui.JedaiUI
import com.anagog.jemaExtension.DeepMs
import java.util.logging.Level

import android.content.pm.PackageManager
import android.os.Bundle
import android.os.Handler
import android.os.HandlerThread
import android.os.Looper
import com.anagog.anagog_flutter.AnagogFlutterPlugin
import com.anagog.anagog_flutter.BuildConfig
import com.anagog.anagog_flutter.exceptions.NoLicenseKeyException
import com.anagog.anagog_flutter.model.Micromoment
import com.anagog.anagog_flutter.model.PluginConfig
import com.anagog.anagog_flutter.plugin_callbacks.*
import com.anagog.anagog_flutter.plugin_utils.*
import com.anagog.anagog_flutter.plugin_utils.PluginUtils.activityTypeMapper

import com.anagog.jedai.common.ApplicationEvent
import com.anagog.jedai.common.config.VariantValue
import com.anagog.jedai.common.poi.Point
import com.anagog.jedai.common.stats.DataType
import com.anagog.jedai.common.stats.Stats
import com.anagog.jedai.core.api.JedAIApiWrapperThreadSafe
import com.anagog.jedai.core.logger.JedAILogger
import com.anagog.jedai.jema.models.JemaUserInteractionEvent
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodChannel
import kotlin.collections.HashMap

private const val TAG = "PluginJemaWrapper"

object JemaWrapper {

    private val mainHandler = Handler(Looper.getMainLooper())
    private val mHandlerThread = HandlerThread("wrapper-main").also {
        it.start()
    }
    private val wrapperHandler = Handler(mHandlerThread.looper)
    private val userDefinedStats = mutableMapOf<String, Stats.Type>()

    private var eventSink: EventChannel.EventSink? = null

    private lateinit var pluginConfig: PluginConfig

    private val pendingEventsQueue = mutableListOf<Map<String, Any?>>()
    private var approvalHandler: PluginNotificationApprovalHandler? = null
    private var campaignTriggeredHandler: PluginCampaignTriggeredHandler? = null
    private var notificationClickedHandler: PluginNotificationClickedHandler? = null
    private var snapshotHandler: PluginReportSnapshotHandler? = null

    val sLogger: JedAILogger = JedAILogger.getLogger(AnagogFlutterPlugin::class.java)

    @JvmStatic
    fun setup(context: Context, config: PluginConfig) {
        this.pluginConfig = config
        val appMetaData = context.packageManager.getApplicationInfo(context.packageName, PackageManager.GET_META_DATA).metaData
        val apiKey = appMetaData.getString(ANAGOG_API_KEY, null) ?: throw NoLicenseKeyException()
        sLogger.fine("API Key: $apiKey")
        Log.i(TAG, "API Key: $apiKey")
        JedAI.setApiKey(apiKey)
        JedAI.getInstance()?.configureCrashReports(this.pluginConfig.enableCrashReports)

        if (BuildConfig.DEBUG) { //Setting JedAI and Jema in debug mode
            pluginConfig.remoteConfigUrl?.let { JedAI.setup(context, it) } ?: JedAI.setup(context)

            if (pluginConfig.reportBaseUrl != null) {
                val usageReportBaseUrl = "${pluginConfig.reportBaseUrl}usage/periodical"
                Log.i(TAG, "Setting usage report base url: $usageReportBaseUrl")
                JedAI.getInstance()?.configureUsageReportUrl(usageReportBaseUrl)
            }

            val clientName = pluginConfig.clientName
            val reportBaseUrl = pluginConfig.reportBaseUrl
            val manifestUrl = pluginConfig.manifestUrl
            if (clientName != null && reportBaseUrl != null && manifestUrl != null) {
                sLogger.fine("setConfig client name: $clientName")
                Log.i(TAG, "setConfig client name: $clientName")
                JedAIJema.setup(clientName = clientName, reportBaseUrl = reportBaseUrl, manifestUrl = manifestUrl)
            } else {
                JedAIJema.setup(context) //Default bundle id
            }
        } else { //JedAI and Jema setup for production
            JedAI.setup(context)
            JedAIJema.setup(context)
        }

        if(pluginConfig.enableInternalLog) {
            setInternalog()
        }

        JedAI.getInstance()?.configurePoi(poiProvidersConfig)
        // JEMA will use this integration logs listener to send log messages that provide
        // important information for the integration of JEMA
        // for example messages about the downloading of campaigns from the server
        JedAI.getInstance()?.enableIntegrationLogs { level, message ->
            Log.i("INTEGRATION", "${level.localizedName} $message")
        }

        initJedaiUi(context)

        JedAI.getInstance()?.setMicrosegmentsSnapshotReportHandler {
            onSnapshotReady(it)
        }

        pluginConfig.let {
            setUserDefinedStats(it)
            JedAI.getInstance()?.setForegroundServiceAlwaysOn(it.foregroundAlwaysOn)
            JedAI.getInstance()?.configureAudienceAnalytics(it.enableAudienceAnalytics)
            runJedAIInForeground(context, it, appMetaData)
        }

        JedAI.getInstance()?.start()
        if (pluginConfig.enableLatestMs) {
            DeepMs.start(context)
        }
        emitEvent(PluginUtils.preparePluginMessage(ON_JEDAI_READY, null))
    }


    private fun setInternalog() {
        JedAI.getInstance()?.setInternal(object : HashMap<String, Any>() {
            init {
                val SECRET = 0x67a8e783a3c11bc2L
                put("Key", SECRET)
                put("LogToConsole", 1)
                put("LogToFiles", 1)
                put("Level", Level.FINEST)
            }
        })
    }

    private fun runJedAIInForeground(context: Context, pluginConfig: PluginConfig, appMetaData: Bundle) {
        Log.i(TAG, "runJedAIInForeground: ")
        val notification = NotificationHelper.setupForegroundNotification(context, pluginConfig, appMetaData)
        JedAI.setForegroundNotification(notification)
    }


    internal fun syncCampaigns() {
        JedAIJema.syncCampaigns(true)
    }

    internal fun simulateCampaign(campaignIdentifier: String?) {
        JedAIJema.simulateCampaign(campaignIdentifier)
    }


    internal fun getUserDefinedStat(name: String, channelResult: MethodChannel.Result) {
        runAsync({
            val result = JedAI.getInstance()?.getStats(name, null, null, null)
            Log.i(TAG, "getUserDefinedStat: result: type: ${result?.metricType}")
            val stat: String? = result?.let {
                when (it.metricType) {
                    DataType.DECIMAL -> it.decimalValue.toString()
                    DataType.INTEGER -> it.integerValue.toString()
                    DataType.STRING -> it.stringValue.toString()
                }
            }
            val success = mapOf<String, Any?>(name to stat)
            Log.i(TAG, "getUserDefinedStat: result: stat: $stat, success: $success")
            success
        }, channelResult)
    }

    internal fun getUserDefinedStats(result: MethodChannel.Result) {
        val operation = {
            val resultMap = mutableMapOf<String, String?>()
            userDefinedStats.forEach {
                val statsType = when (it.value.dataType) {
                    DataType.STRING -> "String"
                    DataType.INTEGER -> "Integer"
                    DataType.DECIMAL -> "Decimal"
                    else -> "String"
                }
                resultMap[it.key] = statsType
            }
            resultMap
        }
        runAsync(operation = operation, channelResult = result)
    }

    private fun initJedaiUi(context: Context) {
        sLogger.fine("initJedaiUi: ")

        val notificationChannel = pluginConfig.notificationChannelId ?: context.getAppName()
        JedaiUI.setup(context = context, notificationChannel = notificationChannel, activity = null)

        JedaiUI.setCampaignNotificationApprovementHandler { userObject ->
            sLogger.fine("setCampaignNotificationApprovalHandler: ")
            approvalHandler?.let {
                val approved = it.onNotificationApproval(event = userObject)
                sLogger.fine("Use app ApprovalHandler: approved: $approved")
                approved
            } ?: run {
                sLogger.fine("No CampaignNotificationApprovalHandler defined.. approval set to true")
                true
            }
        }

        JedaiUI.setCampaignNotificationClickActionHandler { executed, action, userObject ->
            notificationClickedHandler?.let {
                sLogger.fine("Use app ClickActionHandler..")
                it.onNotificationClicked(executed, action, userObject)
            } ?: run {
                sLogger.fine("No CampaignNotificationClickActionHandler defined..")
                handleNotificationExecution(executed, action, userObject)
            }
        }

        JedaiUI.setCampaignTriggerActionHandler { _, userObject ->
            campaignTriggeredHandler?.let {
                sLogger.fine("Use app TriggerActionHandler..")
                it.onCampaignTriggered(userObject)
            } ?: run {
                sLogger.fine("No TriggerActionHandler defined.. Use default plugin handler")
                emitEvent(PluginUtils.preparePluginMessage(ON_EVENT_TRIGGERED, userObject))

            }
        }
    }

    private fun onSnapshotReady(snapshot: Map<String, VariantValue>) {
        val payload = mutableMapOf<String, Map<String, Any?>>()
        snapshot.forEach { element ->
            payload[element.key] = mapOf(element.value.toValue())
        }
        snapshotHandler?.onReportSnapshot(payload) ?: run {
            emitEvent(PluginUtils.preparePluginMessage(ON_SNAPSHOT_REPORT, payload))
        }
    }


    private fun handleNotificationExecution(executed: Boolean, action: String?, userObject: Map<String, Any>) {
        sLogger.fine("handleNotificationExecution: $userObject")
        val campaign = mutableMapOf<String, Any>()
        campaign.putAll(userObject)
        campaign["executed"] = executed
        val event: Map<String, Any?> = PluginUtils.preparePluginMessage(ON_NOTIFICATION_CLICKED, campaign)
        emitEvent(event)
    }

    private val poiProvidersConfig = listOf(
        // home/work (PoiAlgorithmType4Config) is added to the list of providers explicitly
        PoiAlgorithmType1Config.Builder().build()
    )

    private fun setUserDefinedStats(config: PluginConfig) {
        sLogger.fine("setUserDefinedStats: ")
        val configUserDefinedStats = config.getUserDefinedStats()
        userDefinedStats.clear()
        userDefinedStats.putAll(configUserDefinedStats)
        if (config.enableLatestMs) {
            DeepMs.addDeepMs(configUserDefinedStats)
        }
        JedAI.getInstance()?.setupUserDefinedStats(configUserDefinedStats)
    }


    internal fun getUserDefinedString(key: String, channelResult: MethodChannel.Result) {
        runAsync({
            val result = JedAI.getInstance()?.getStats(key, null, null, null)?.stringValue
            Log.i(TAG, "getUserDefinedString: key: $key, result: $result")
            result
        }, channelResult)
    }

    internal fun setUserDefinedString(name: String, value: String?) {
        runAsync({
            JedAI.getInstance()?.setUserDefinedStringStats(name, value, null, null)
        })
    }

    internal fun setUserDefinedInteger(name: String, value: Int?) {
        Log.i(TAG, "setUserDefinedInteger: name: $name, value: $value")
        runAsync({
            JedAI.getInstance()?.setUserDefinedIntegerStats(name, value, null, null)
        })
    }

    internal fun getUserDefinedInteger(name: String, channelResult: MethodChannel.Result) {
        Log.i(TAG, "getUserDefinedInt: name: $name")
        runAsync({
            JedAI.getInstance()?.getStats(name, null, null, null)?.integerValue
        }, channelResult)
    }

    internal fun setUserDefinedDecimal(name: String, value: Double?) {
        runAsync({
            JedAI.getInstance()?.setUserDefinedDecimalStats(name, value, null, null)
        })
    }

    internal fun getUserDefinedDecimal(name: String, channelResult: MethodChannel.Result) {
        runAsync({
            JedAI.getInstance()?.getStats(name, null, null, null)?.decimalValue
        }, channelResult)
    }


    internal fun fireMicromoment(microMoment: Micromoment) {
        Log.i(TAG, "fireMicromoment: micromoment: $microMoment")
        val identifier = microMoment.identifier
        val appEvent = ApplicationEvent(identifier)

        microMoment.textParameters.forEach {
            appEvent.addParameter(it.key, it.value)
        }

        microMoment.numericParameters.forEach { entry ->
            appEvent.addParameter(entry.key, (entry.value).toFloat())
        }

        microMoment.locationParameters.forEach { entry ->
            val point = Point(entry.value.latitude, entry.value.longitude)
            appEvent.addParameter(entry.key, point)
        }

        Log.i(TAG, "appEvent: $appEvent, ${appEvent.textParameters}, ${appEvent.numericParameters}, ${appEvent.locationParameters}")
        postApplicationEvent(appEvent)
    }

    internal fun share(context: Context, result: MethodChannel.Result) {
        JedAIDebugging.getDebuggingDataShareIntent(context){
            it.flags = Intent.FLAG_ACTIVITY_NEW_TASK
            context.startActivity(it)
            result.success(null)
        }
    }

    internal fun getVersion(channelResult: MethodChannel.Result) {
        runAsync({
            JedAI.getInstance()?.version
        }, channelResult)
    }

    private fun postApplicationEvent(event: ApplicationEvent) {
        JedAI.getInstance()?.postApplicationEvent(event)
    }

    internal fun setUserInteractionEvent(campaignIdentifier: String, activityType: String) {
        Log.i(TAG, "About to setUserInteractionEvent: campaignIdentifier: $campaignIdentifier, activityType: $activityType")
        activityTypeMapper(activityType)?.let {
            runAsync({
                val event = JemaUserInteractionEvent(campaignIdentifier, it)
                Log.i(TAG, "JemaUserInteractionEvent: $event")
                JedAIJema.sendEvent(event, mapOf())
            })
        }
    }

    internal fun requestMicrosegmentsSnapshotReport() {
        JedAI.getInstance()?.requestMicrosegmentsSnapshotReport()
    }

    internal fun forceScheduleReports() {
        Log.i(TAG, "forceScheduleReports: ")
        JedAIApiWrapperThreadSafe.getInstance().forceReportSend()
    }

    internal fun getCampaigns(channelResult: MethodChannel.Result? = null) {
        val operation = {
            val resultList = mutableListOf<Map<String, String>>()
            JedAIJema.getAllCampaigns().forEach {
                Log.i(TAG, "getCampaigns: $it")
                resultList.add(mapOf("title" to it.title, "identifier" to it.identifier))
            }
            Log.i(TAG, "getCampaigns: $resultList")
            resultList
        }
        runAsync( operation = operation , channelResult = channelResult)
    }

    internal fun isJedAIReady(channelResult: MethodChannel.Result? = null) {
        val operation = {
            JedAIApiWrapperThreadSafe.getInstance().jedAIHandler.looper.thread.isAlive
        }
        runAsync(operation = operation, channelResult = channelResult)
    }


    /********************* Set Plugin Handlers **************************/
    @JvmStatic
    fun setNotificationApprovalHandler(handler: PluginNotificationApprovalHandler) {
        this.approvalHandler = handler
    }

    @JvmStatic
    fun setCampaignTriggeredHandler(handler: PluginCampaignTriggeredHandler) {
        this.campaignTriggeredHandler = handler
    }

    @JvmStatic
    fun setNotificationClickHandler(handler: PluginNotificationClickedHandler) {
        this.notificationClickedHandler = handler
    }

    @JvmStatic
    fun setReportSnapshotHandler(handler: PluginReportSnapshotHandler) {
        this.snapshotHandler = handler
    }

    /********************************/

    private fun emitEvent(event: Map<String, Any?>) {
        Log.i(TAG, "emitEvent: $event")
        mainHandler.post {
            eventSink?.success(event) ?: run {
                Log.i(TAG, "eventSink not initialized: ")
                sLogger.fine("eventSink not initialized: ")
                pendingEventsQueue.add(event)
            }
        }
    }


    private fun runAsync(operation: () -> Any?, channelResult: MethodChannel.Result? = null) {
        wrapperHandler.post {
            val result = operation.invoke()
            channelResult?.let {
                mainHandler.post {
                    it.success(result)
                }
            }
        }
    }

    /********************************/

    internal fun setEventSink(sink: EventChannel.EventSink) {
        Log.i(TAG, "setEventSync: ")
        if (eventSink == null) {
            eventSink = sink
        }

        pendingEventsQueue.forEach {
            Log.i(TAG, "flush pendingEventsQueue: $it")
            eventSink?.success(it) //Deliver undelivered campaign
            pendingEventsQueue.remove(it)
        }
        pendingEventsQueue.clear()
    }

    internal fun closeEventSync() {
        Log.i(TAG, "closeEventSync: ")
        eventSink = null
    }

}