//
//  JemaAPI.swift
//
//  Copyright © 2022 Anagog Ltd. All rights reserved.
//

import Foundation
import JedAIKit
import JedAIMetricsKit
import JedAIJEMAKit
import JedAIReportKit
import CoreLocation

class JemaAPI {
    
    static let shared = JemaAPI()
        
    // MARK: - Initialization

    private init() {}
    
    // MARK: - Micromoments
    
    func fireMicromoment(_ micromoment: Dictionary<String, Any>) {
        guard let identifier = micromoment["identifier"] as? String else {
            return
        }
        var applicationEvent = ApplicationEvent(identifier)
        
        if let textParameters = micromoment["textParameters"] as? Dictionary<String, String> {
            for (name, text) in textParameters {
                applicationEvent = applicationEvent.addParameter(name: name, text: text)
            }
        }
        if let numericParameters = micromoment["numericParameters"] as? Dictionary<String, Float> {
            for (name, number) in numericParameters {
                applicationEvent = applicationEvent.addParameter(name: name, number: number)
            }
        }
        if let locationParameters = micromoment["locationParameters"] as? Dictionary<String, Dictionary<String, CLLocationDegrees>> {
            for (name, rawLocation) in locationParameters {
                guard let latitude = rawLocation["latitude"], let longitude = rawLocation["longitude"] else {
                    continue
                }
                let location = CLLocationCoordinate2D(latitude: latitude, longitude: longitude)
                applicationEvent = applicationEvent.addParameter(name: name, location: location)
            }
        }
        applicationEvent.post()
    }
    
    // MARK: - Microsegments
    
    func getUserDefinedMetrics() -> [String: String] {
        var result = [String: String]()
        for (metricName, metricType) in JedAIMetrics.shared.userDefinedMetrics {
            switch metricType {
            case .string:
                result[metricName] = "String"
            case .decimal:
                result[metricName] = "Decimal"
            case .integer:
                result[metricName] = "Integer"
            default:
                break
            }
        }
        return result
    }
    
    func setUserDefinedString(_ customName: String, value: String) {
        JedAIMetricsModel.Metric.UserDefinedString.set(customName: customName, value: value)
    }
    
    func setUserDefinedInteger(_ customName: String, value: Int) {
        JedAIMetricsModel.Metric.UserDefinedInteger.set(customName: customName, value: value)
    }
    
    func setUserDefinedDecimal(_ customName: String, value: Double) {
        JedAIMetricsModel.Metric.UserDefinedDecimal.set(customName: customName, value: value)
    }
    
    func getUserDefinedString(_ customName: String) -> String {
        return JedAIMetricsModel.Metric.UserDefinedString.get(customName: customName)
    }
    
    func getUserDefinedInteger(_ customName: String) -> Int {
        return JedAIMetricsModel.Metric.UserDefinedInteger.get(customName: customName)
    }
    
    func getUserDefinedDecimal(_ customName: String) -> Double {
        return JedAIMetricsModel.Metric.UserDefinedDecimal.get(customName: customName)
    }
    
    // MARK: - Campaigns
    
    func downloadCampaigns() {
        DispatchQueue.global().async {
            JedAICommandEvent(command: "enforceJEMASync").post()
        }
    }
    
    func simulateCampaign(_ campaignIdentifier: String? = nil) {
        JedAIJEMA.shared.simulateCampaign(campaignIdentifier)
    }
    
    func getCampaigns() -> [[String: String]] {
        return JedAIJEMA.shared.loadActiveCampaigns().map({ campaign in
            return [
                "title": campaign.title,
                "identifier": campaign.identifier
            ]
        })
    }
    
    // MARK: - Reprots
    
    func forceScheduleReports() {
        JedAICommandEvent(command: "enforceReportGeneration").post()
    }
    
    func requestMicrosegmentsSnapshotReport() {
        JedAIReport.shared.requestMicrosegmentsSnapshotReport()
    }
    
    // MARK: - JedAI
    
    func isJedAIReady() -> Bool {
        JedAI.shared.state == JedAIState.active
    }
    
    func share(completion: @escaping () -> Void) {
        DispatchQueue.main.async {
            ShareService.shareDatabaseAndLogs { _, _, _, _ in
                completion()
            }
        }
    }
    
    func getVersion() -> String {
        return JedAI.shared.api.version
    }
    
    // MARK: - Other
    
    @discardableResult
    func sendUserInteractionEvent(campaignIdentifier: String, triggerType triggerTypeString: String) -> Bool {
        let triggerType: JEMAUserInteractionEvent.TriggerType
        switch triggerTypeString {
        case "DISMISSED":
            triggerType = .notificationDismissed
        case "TIMEOUT":
            triggerType = .notificationTimeout
        case "TRIGGERED":
            triggerType = .acceptedTrigger
        case "CONVERSION":
            triggerType = .conversion
        case "CLICKED":
            triggerType = .notificationClick
        default:
            return false
        }
        JEMAUserInteractionEvent(campaignIdentifier: campaignIdentifier, triggerType: triggerType).post()
        return true
    }
    
}
